@@ -19,7 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
 *)
 
-(** SVG types with variants. (See also {!Svg.M}) *)
+(** SVG types with variants, goes with {!Svg_sigs.T}. *)
 
 (** This module defines basic data types for data, attributes
     and element occuring in SVG documents.
@@ -155,7 +155,7 @@ type animation_timing_attr =
   ]
 
 type animation_value_attr =
-  [ | `CalcMode | `Values | `KeyTimes | `KeySplines | `From | `To | `By
+  [ | `CalcMode | `Valuesanim | `KeyTimes | `KeySplines | `From | `To | `By
   ]
 
 type animation_addition_attr = [ | `Additive | `Accumulate ]
@@ -273,7 +273,7 @@ open Unit
 type coord = length
 type number = float
 type number_optional_number = (number * (number option))
-type percentage = int
+type percentage = float
 type strings = string list
 
 type color = string
@@ -290,12 +290,12 @@ type paint =
 
 (* Transformation *)
 type transform =
-  | Matrix of (float * float * float * float * float * float)
-  | Translate of (float * (float option))
-  | Scale of (float * (float option))
-  | Rotate of (angle * ((float * float) option))
-  | SkewX of angle
-  | SkewY of angle
+  [ `Matrix of (float * float * float * float * float * float)
+  | `Translate of (float * (float option))
+  | `Scale of (float * (float option))
+  | `Rotate of (angle * ((float * float) option))
+  | `SkewX of angle
+  | `SkewY of angle ]
 
 type spacestrings = string list
 type commastrings = string list
@@ -893,8 +893,8 @@ type glyphref_attr =
     | `Style
     | `X
     | `Y
-    | `Dx_single
-    | `Dy_single
+    | `Dx
+    | `Dy
     | `GlyphRef
     | `Format
     | `Xlink_href
@@ -1287,7 +1287,7 @@ type fecomposite_attr =
     | `Style
     | `In
     | `In2
-    | `Operator
+    | `OperatorComposite
     | `K1
     | `K2
     | `K3
@@ -1420,7 +1420,7 @@ type femorphology_attr =
     | core_attr
     | presentation_attr
     | filter_primitive_attr
-    | `Operator_morphology
+    | `OperatorMorphology
     | `Class
     | `Style
     | `In
@@ -1497,7 +1497,7 @@ type feturbulence_attr =
     | `NumOctaves
     | `Seed
     | `StitchTiles
-    | `Type_stitch
+    | `TypeStitch
   ]
 
 type cursor = [ | `Cursor ]
@@ -1693,7 +1693,7 @@ type animatetransform_attr =
     | animation_value_attr
     | animation_addition_attr
     | `ExternalResourcesRequired
-    | `Typeanimatecolor
+    | `Typeanimatetransform
   ]
 
 type font = [ | `Font ]
@@ -1825,12 +1825,12 @@ type vkern = [ | `Vkern ]
 (* nullary *)
 type vkern_attr = [ | core_attr | `U1 | `G1 | `U2 | `G2 | `K ]
 
-type fontface = [ | `Font_Face ]
+type font_face = [ | `Font_Face ]
 
 (* nullary *)
-type fontface_content = [ | descriptive_element | `Font_Face_Src ]
+type font_face_content = [ | descriptive_element | `Font_Face_Src ]
 
-type fontface_attr =
+type font_face_attr =
   [
     | core_attr
     | `Font_Family
@@ -1868,29 +1868,29 @@ type fontface_attr =
     | `OverlineThickness
   ]
 
-type fontfacesrc = [ | `Font_Face_Src ]
+(* star *)
+type font_face_src = [ | `Font_Face_Src ]
+
+type font_face_src_content = [ | `Font_Face_Name | `Font_Face_Uri ]
+
+type font_face_src_attr = core_attr
 
 (* star *)
-type fontfacesrc_content = [ | `Font_Face_Name | `Font_Face_Uri ]
+type font_face_uri = [ | `Font_Face_Uri ]
 
-type fontfacesrc_attr = core_attr
+type font_face_uri_content = [ | `Font_Face_Format ]
 
-type fontfaceuri = [ | `Font_Face_Uri ]
-
-(* star *)
-type fontfaceuri_content = [ | `Font_Face_Format ]
-
-type fontfaceuri_attr = [ | core_attr | xlink_attr | `Xlink_href ]
-
-type fontfaceformat = [ | `Font_Face_Format ]
+type font_face_uri_attr = [ | core_attr | xlink_attr | `Xlink_href ]
 
 (* nullary *)
-type fontfaceformat_attr = [ | core_attr | `String ]
+type font_face_format = [ | `Font_Face_Format ]
 
-type fontfacename = [ | `Font_Face_Name ]
+type font_face_format_attr = [ | core_attr | `String ]
 
 (* nullary *)
-type fontfacename_attr = [ | core_attr | `Name ]
+type font_face_name = [ | `Font_Face_Name ]
+
+type font_face_name_attr = [ | core_attr | `Name ]
 
 type metadata = [ | `Metadata ]
 
@@ -1913,3 +1913,145 @@ type foreignobject_attr =
     | `Width
     | `Height
   ]
+
+type alignment_baseline =
+  [ `After_edge
+  | `Alphabetic
+  | `Auto
+  | `Baseline
+  | `Before_edge
+  | `Central
+  | `Hanging
+  | `Ideographic
+  | `Inherit
+  | `Mathematical
+  | `Middle
+  | `Text_after_edge
+  | `Text_before_edge ]
+
+type dominant_baseline =
+  [ `Auto
+  | `Use_script
+  | `No_change
+  | `Reset_size
+  | `Ideographic
+  | `Alphabetic
+  | `Hanging
+  | `Mathematical
+  | `Central
+  | `Middle
+  | `Text_after_edge
+  | `Text_before_edge
+  | `Inherit
+  ]
+
+type in_value =
+  [ `SourceGraphic
+  | `SourceAlpha
+  | `BackgroundImage
+  | `BackgroundAlpha
+  | `FillPaint
+  | `StrokePaint
+  | `Ref of string ] [@@reflect.total_variant]
+
+type offset =
+  [ `Number of number
+  | `Percentage of percentage ]
+
+type big_variant =
+  [ `A
+  | `Absolute_colorimetric
+  | `Align
+  | `Always
+  | `Atop
+  | `Arithmetic
+  | `Auto
+  | `B
+  | `Bever
+  | `Blink
+  | `Butt
+  | `CSS
+  | `Darken
+  | `Default
+  | `Dilate
+  | `Disable
+  | `Discrete
+  | `Duplicate
+  | `End
+  | `Erode
+  | `Exact
+  | `FractalNoise
+  | `Freeze
+  | `HueRotate
+  | `G
+  | `Gamma
+  | `GeometricPrecision
+  | `H
+  | `Identity
+  | `In
+  | `Inherit
+  | `Initial
+  | `Isolated
+  | `Lighten
+  | `Line_through
+  | `Linear
+  | `LuminanceToAlpha
+  | `Magnify
+  | `Matrix
+  | `Medial
+  | `Middle
+  | `Miter
+  | `Multiply
+  | `Never
+  | `New
+  | `None
+  | `Normal
+  | `NoStitch
+  | `ObjectBoundingBox
+  | `OnLoad
+  | `OnRequest
+  | `OptimizeLegibility
+  | `OptimizeSpeed
+  | `Other
+  | `Out
+  | `Over
+  | `Overline
+  | `Paced
+  | `Pad
+  | `Perceptual
+  | `Preserve
+  | `R
+  | `Reflect
+  | `Remove
+  | `Repeat
+  | `Replace
+  | `Relative_colorimetric
+  | `Rotate
+  | `Round
+  | `Saturate
+  | `Saturation
+  | `Scale
+  | `Screen
+  | `SkewX
+  | `SkewY
+  | `Spacing
+  | `SpacingAndGlyphs
+  | `Spline
+  | `Square
+  | `Start
+  | `Stitch
+  | `Stretch
+  | `StrokeWidth
+  | `Sum
+  | `Table
+  | `Terminal
+  | `Translate
+  | `Turbulence
+  | `Underline
+  | `UserSpaceOnUse
+  | `V
+  | `WhenNotActive
+  | `Wrap
+  | `XML
+  | `Xor
+  ]
