@@ -48,25 +48,48 @@ let encode_unsafe_char_and_at s =
   Buffer.contents b
 
 let compose_decl ?(version = "1.0") ?(encoding = "UTF-8") () =
-  "<?xml version=\"" ^ version ^ "\" encoding=\"" ^ encoding ^ "\"?>\n"
+  Format.sprintf
+    {|<?xml version="%s" encoding="%s"?>\n|}
+    version encoding
 
 let compose_doctype dt args =
-  "<!DOCTYPE " ^ dt
-  ^ (if args = []
-     then ""
-     else
-       " PUBLIC " ^
-       String.concat " " (List.map (fun a -> "\"" ^ a ^ "\"") args)) ^ ">"
+  let pp_args fmt = function
+    | [] -> ()
+    | l ->
+      Format.fprintf fmt " PUBLIC %a"
+        (Format.pp_print_list ~pp_sep:Format.pp_print_space
+           (fun fmt -> Format.fprintf fmt "\"%s\""))
+        l
+  in
+  Format.asprintf
+    "<!DOCTYPE %s%a>"
+    dt
+    pp_args args
 
+let re_end_comment = Re.(compile @@ alt [
+  seq [ bos ; str ">" ] ;
+  seq [ bos ; str "->" ] ;
+  str "-->" ;
+  str "--!>" ;
+])
+let escape_comment s =
+  let f g = match Re.Group.get g 0 with
+    | ">" -> "&gt;"
+    | "->" -> "-&gt;"
+    | "-->" -> "--&gt;"
+    | "--!>" -> "--!&gt;"
+    | s -> s
+  in
+  Re.replace ~all:true re_end_comment ~f s
 
 (* copied form js_of_ocaml: compiler/javascript.ml *)
-let string_of_number v =
+let pp_number fmt v =
   if v = infinity
-  then "Infinity"
+  then Format.pp_print_string fmt "Infinity"
   else if v = neg_infinity
-  then "-Infinity"
+  then Format.pp_print_string fmt "-Infinity"
   else if v <> v
-  then "NaN"
+  then Format.pp_print_string fmt "NaN"
   else
     let vint = int_of_float v in
     (* compiler 1000 into 1e3 *)
@@ -77,19 +100,21 @@ let string_of_number v =
         then div (n/10) (succ i)
         else
         if i > 2
-        then Printf.sprintf "%de%d" n i
-        else string_of_int vint in
+        then Format.fprintf fmt "%de%d" n i
+        else Format.pp_print_int fmt vint in
       div vint 0
     else
       let s1 = Printf.sprintf "%.12g" v in
       if v = float_of_string s1
-      then s1
+      then Format.pp_print_string fmt s1
       else
         let s2 = Printf.sprintf "%.15g" v in
         if v = float_of_string s2
-        then s2
-        else  Printf.sprintf "%.18g" v
+        then Format.pp_print_string fmt s2
+        else  Format.fprintf fmt "%.18g" v
 
+let string_of_number v =
+  Format.asprintf "%a" pp_number v
 
 module Utf8 = struct
   type utf8 = string
@@ -98,7 +123,7 @@ module Utf8 = struct
     let warn = ref false in
     let buffer = Buffer.create (String.length src) in
     Uutf.String.fold_utf_8
-      (fun _ i d ->
+      (fun _ _ d ->
          match d with
          | `Uchar code -> Uutf.Buffer.add_utf_8 buffer code
          | `Malformed _ ->
@@ -127,38 +152,41 @@ module Utf8 = struct
       let warn = ref false in
       let buffer = Buffer.create (String.length src) in
       Uutf.String.fold_utf_8
-        (fun _ i d ->
+        (fun _ _ d ->
            match d with
-           | `Uchar 34 ->
-               Buffer.add_string buffer "&quot;"
-           | `Uchar 38 ->
-               Buffer.add_string buffer "&amp;"
-           | `Uchar 60 ->
-               Buffer.add_string buffer "&lt;"
-           | `Uchar 62 ->
-               Buffer.add_string buffer "&gt;"
-           | `Uchar code ->
-               let u =
-                 (* Illegal characters in html
-                  http://en.wikipedia.org/wiki/Character_encodings_in_HTML
-                  http://www.w3.org/TR/html5/syntax.html *)
-                 if (* A. control C0 *)
-                   (code <= 31 && code <> 9 && code <> 10 && code <> 13)
-                   (* B. DEL + control C1
-                    - invalid in html
-                    - discouraged in xml;
-                      except 0x85 see http://www.w3.org/TR/newline
-                      but let's discard it anyway *)
-                   || (code >= 127 && code <= 159)
-                   (* C. UTF-16 surrogate halves : already discarded by uutf *)
-                   (* || (code >= 0xD800 && code <= 0xDFFF) *)
-                   (* D. BOOM related *)
-                   || code land 0xFFFF = 0xFFFE
-                   || code land 0xFFFF = 0xFFFF
-                 then (warn:=true; Uutf.u_rep)
-                 else code
-               in
-               Uutf.Buffer.add_utf_8 buffer u
+           | `Uchar u ->
+               begin match Uchar.to_int u with
+               | 34 ->
+                   Buffer.add_string buffer "&quot;"
+               | 38 ->
+                   Buffer.add_string buffer "&amp;"
+               | 60 ->
+                   Buffer.add_string buffer "&lt;"
+               | 62 ->
+                   Buffer.add_string buffer "&gt;"
+               | code ->
+                   let u =
+                     (* Illegal characters in html
+                        http://en.wikipedia.org/wiki/Character_encodings_in_HTML
+                        http://www.w3.org/TR/html5/syntax.html *)
+                     if (* A. control C0 *)
+                       (code <= 31 && code <> 9 && code <> 10 && code <> 13)
+                       (* B. DEL + control C1
+                          - invalid in html
+                          - discouraged in xml;
+                          except 0x85 see http://www.w3.org/TR/newline
+                          but let's discard it anyway *)
+                       || (code >= 127 && code <= 159)
+                       (* C. UTF-16 surrogate halves : already discarded
+                          by uutf || (code >= 0xD800 && code <= 0xDFFF) *)
+                       (* D. BOM related *)
+                       || code land 0xFFFF = 0xFFFE
+                       || code land 0xFFFF = 0xFFFF
+                     then (warn:=true; Uutf.u_rep)
+                     else u
+                   in
+                   Uutf.Buffer.add_utf_8 buffer u
+               end
            | `Malformed _ ->
                Uutf.Buffer.add_utf_8 buffer Uutf.u_rep;
                warn:=true)
@@ -169,10 +197,130 @@ module Utf8 = struct
 
 end
 
+module type TagList = sig val emptytags : string list end
+
+(** Format based printers *)
+
+let pp_noop _fmt _ = ()
+
+module Make_fmt
+    (Xml : Xml_sigs.Iterable)
+    (I : TagList) =
+struct
+  open Xml
+
+  module S = Set.Make(String)
+  let is_emptytag = match I.emptytags with
+    | [] -> fun _ -> false
+    | l ->
+      let set = List.fold_left (fun s x -> S.add x s) S.empty l in
+      fun x -> S.mem x set
+
+  let pp_encode encode fmt s =
+    Format.pp_print_string fmt (encode s)
+
+  let pp_sep = function
+    | Space -> fun fmt () -> Format.pp_print_char fmt ' '
+    | Comma -> fun fmt () -> Format.pp_print_string fmt ", "
+
+  let pp_attrib_value encode fmt a = match acontent a with
+    | AFloat f -> Format.fprintf fmt "\"%a\"" pp_number f
+    | AInt i -> Format.fprintf fmt "\"%d\"" i
+    | AStr s -> Format.fprintf fmt "\"%s\"" (encode s)
+    | AStrL (sep, slist) ->
+      Format.fprintf fmt "\"%a\""
+        (Format.pp_print_list ~pp_sep:(pp_sep sep) (pp_encode encode)) slist
+
+  let pp_attrib encode fmt a =
+    Format.fprintf fmt
+      " %s=%a" (aname a) (pp_attrib_value encode) a
+
+  let pp_attribs encode =
+    Format.pp_print_list ~pp_sep:pp_noop (pp_attrib encode)
+
+  let pp_closedtag encode fmt tag attrs =
+    if is_emptytag tag then
+      Format.fprintf fmt "<%s%a/>" tag (pp_attribs encode) attrs
+    else
+      Format.fprintf fmt "<%s%a></%s>" tag (pp_attribs encode) attrs tag
+
+  let rec pp_tag encode fmt tag attrs taglist =
+    match taglist with
+    | [] -> pp_closedtag encode fmt tag attrs
+    | _ ->
+      Format.fprintf fmt "<%s%a>%a</%s>"
+        tag
+        (pp_attribs encode) attrs
+        (pp_elts encode) taglist
+        tag
+
+  and pp_elt encode fmt elt = match content elt with
+    | Comment texte ->
+      Format.fprintf fmt "<!--%s-->" (escape_comment texte)
+
+    | Entity e ->
+      Format.fprintf fmt "&%s;" e
+
+    | PCDATA texte ->
+      pp_encode encode fmt texte
+
+    | EncodedPCDATA texte ->
+      Format.pp_print_string fmt texte
+
+    | Node (name, xh_attrs, xh_taglist) ->
+      pp_tag encode fmt name xh_attrs xh_taglist
+
+    | Leaf (name, xh_attrs) ->
+      pp_closedtag encode fmt name xh_attrs
+
+    | Empty -> ()
+
+  and pp_elts encode =
+    Format.pp_print_list ~pp_sep:pp_noop (pp_elt encode)
+
+  let pp ?(encode=encode_unsafe_char) () =
+    pp_elt encode
+
+end
+
+module Make_typed_fmt
+    (Xml : Xml_sigs.Iterable)
+    (Typed_xml : Xml_sigs.Typed_xml with module Xml := Xml) =
+struct
+
+  module P = Make_fmt(Xml)(Typed_xml.Info)
+
+  (* Add an xmlns tag on the html element if it's not already present *)
+  let prepare_document doc =
+    let doc = Typed_xml.doc_toelt doc in
+    match Xml.content doc with
+    | Xml.Node (n, a, c) ->
+      let a =
+        if List.exists (fun a -> Xml.aname a = "xmlns") a
+        then a
+        else Xml.string_attrib "xmlns" Typed_xml.Info.namespace :: a
+      in
+      Xml.node ~a n c
+    | _ -> doc
+
+  let pp_elt ?(encode=encode_unsafe_char) () fmt foret =
+    P.pp_elt encode fmt (Typed_xml.toelt foret)
+
+  let pp ?(encode = encode_unsafe_char) ?advert () fmt doc =
+    Format.pp_print_string fmt Typed_xml.Info.doctype ;
+
+    begin match advert with
+      | Some s -> Format.fprintf fmt "<!-- %s -->\n" s
+      | None -> Format.pp_print_newline fmt ()
+    end ;
+
+    P.pp_elt encode fmt (prepare_document doc)
+
+end
 
 module Make
     (Xml : Xml_sigs.Iterable)
-    (F : sig val emptytags : string list end)
+    (I : TagList)
     (O : Xml_sigs.Output) =
 struct
 
@@ -201,10 +349,8 @@ struct
       O.put (" "^ attrib_to_string encode attr)
       ++ xh_print_attrs encode queue
 
-  and xh_print_text texte = O.put texte
-
   and xh_print_closedtag encode tag attrs =
-    if F.emptytags = [] || List.mem tag F.emptytags
+    if I.emptytags = [] || List.mem tag I.emptytags
     then
       (O.put ("<"^tag)
        ++ xh_print_attrs encode attrs
@@ -267,7 +413,7 @@ end
 
 module Make_typed
     (Xml : Xml_sigs.Iterable)
-    (Typed_xml : Xml_sigs.Iterable_typed_xml with module Xml := Xml)
+    (Typed_xml : Xml_sigs.Typed_xml with module Xml := Xml)
     (O : Xml_sigs.Output) =
 struct
 
@@ -306,11 +452,9 @@ end
 
 module Make_simple
     (Xml : Xml_sigs.Iterable)
-    (I : sig val emptytags : string list end) =
+    (I : TagList) =
 struct
 
-  type elt = Xml.elt
-  type out = unit
   let print_list ~output =
     let module M = Make(Xml)(I)(Simple_output(struct let put = output end)) in
     M.print_list
@@ -322,10 +466,6 @@ module Make_typed_simple
     (Typed_xml : Xml_sigs.Typed_xml with  module Xml := Xml) =
 struct
 
-  type out = unit
-  type 'a elt = 'a Typed_xml.elt
-  type doc = Typed_xml.doc
-
   let print_list ~output =
     let module M =
       Make_typed(Xml)(Typed_xml)(Simple_output(struct let put = output end)) in
