@@ -18,14 +18,18 @@
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
 *)
 
-module type Wrapped = sig
+(** XML Signatures. *)
 
-  type 'a wrap
-  type 'a list_wrap
+module type T = sig
+
+  module W : Xml_wrap.T
+
+  type 'a wrap = 'a W.t
+  type 'a list_wrap = 'a W.tlist
 
   type uri
-  val string_of_uri : uri -> string
-  val uri_of_string : string -> uri
+  val string_of_uri : (uri, string) W.ft
+  val uri_of_string : (string, uri) W.ft
 
   type aname = string
   type event_handler
@@ -64,11 +68,10 @@ module type Wrapped = sig
 
 end
 
-module type T = Wrapped with type 'a wrap = 'a
-                         and type 'a list_wrap = 'a list
+module type NoWrap = T with module W = Xml_wrap.NoWrap
 module type Iterable = sig
 
-  include T
+  include NoWrap
 
   type separator = Space | Comma
 
@@ -114,19 +117,7 @@ end
 
 module type Typed_xml = sig
 
-  module Xml : T
-  module Info : Info
-
-  type 'a elt
-  type doc
-  val toelt : 'a elt -> Xml.elt
-  val doc_toelt : doc -> Xml.elt
-
-end
-
-module type Iterable_typed_xml = sig
-
-  module Xml : Iterable
+  module Xml : NoWrap
   module Info : Info
 
   type 'a elt
@@ -184,3 +175,45 @@ module type Typed_simple_printer = sig
     doc -> unit
 
 end
+
+module type Pp = sig
+
+  type elt
+
+(** [pp ()] is a {!Format} printer for untyped XML.
+
+    It can be used in combination with ["%a"]. For example, to get a string:
+    {[let s = Format.asprintf "%a" (pp ()) my_xml]}
+
+    A custom encoding function can be provided with the [~encode] argument.
+    Various implementations of [encode] are available in {!Xml_print}.
+*)
+  val pp:
+    ?encode:(string -> string) -> unit -> Format.formatter -> elt -> unit
+end
+
+module type Typed_pp = sig
+
+  type 'a elt
+  type doc
+
+(** [pp_elt ()] is a {!Format} printer for individual elements.
+
+    A custom encoding function can be provided with the [~encode] argument.
+    Various implementations of [encode] are available in {!Xml_print}.
+*)
+  val pp_elt :
+    ?encode:(string -> string) -> unit -> Format.formatter -> 'a elt -> unit
+
+(** [pp ()] is a {!Format} printer for complete documents.
+
+    It can be used in combination with ["%a"]. For example, to get a string:
+    {[let s = Format.asprintf "%a" (pp ()) my_document]}
+
+    A custom encoding function can be provided with the [~encode] argument.
+    Various implementations of [encode] are available in {!Xml_print}.
+*)
+  val pp:
+    ?encode:(string -> string) -> ?advert:string -> unit -> Format.formatter -> doc -> unit
+
+end
