@@ -19,16 +19,22 @@
 
 module type T = sig
   type 'a t
-  type 'a tlist
   val return : 'a -> 'a t
-  val fmap : ('a -> 'b) -> 'a t -> 'b t
 
+  type (-'a, 'b) ft
+  val fmap : ('a, 'b) ft -> 'a t -> 'b t
+
+  type 'a tlist
   val nil : unit -> 'a tlist
   val singleton : 'a t -> 'a tlist
   val cons : 'a t -> 'a tlist -> 'a tlist
   val append : 'a tlist -> 'a tlist -> 'a tlist
-  val map : ('a -> 'b ) -> 'a tlist -> 'b tlist
+  val map : ('a, 'b) ft -> 'a tlist -> 'b tlist
 end
 
-module NoWrap : T with type 'a t = 'a
-                   and type 'a tlist = 'a list
+module type NoWrap =
+  T with type 'a t = 'a
+     and type 'a tlist = 'a list
+     and type (-'a, 'b) ft = 'a -> 'b
+
+module NoWrap : NoWrap
