@@ -3,7 +3,9 @@ let arg = ref false
 
 let answer_elem ~attrib s =
   Printf.printf
-    "%s\t%s\t%s\n%!"
+    "Original name: \t%s\n\
+      Ocaml name: \t%s\n\
+      Poly variant: \t%s\n"
     s
     (if attrib then Tyxml_name.attrib s else Tyxml_name.ident s)
     (Tyxml_name.polyvar s)
@@ -15,7 +17,7 @@ let spec = [
 ]
 
 let usage =
-  "Accepts HTML attributes and elements and returns their tyxml names.
-Names without option are treated as elements."
+  "Accepts HTML attributes and elements and returns their tyxml names.\
+   Names without option are treated as elements."
 
 let () = Arg.parse spec (answer_elem ~attrib:false) usage
